#!/usr/bin/perl -w
##
###########################################################################
## 
## 
## NLANR Multicast Beacon
## http://dast.nlanr.net/Projects/Beacon
## 
## Copyright (c) 1999,2000,2001,2002,  The Board of Trustees of the University of Illinois 
## All Rights Reserved.
## 
## Developed by:                         Kai Chen
##                                     Mitch Kutzko
##                                     Tony Rimovsky
## 
##                      National Center for Supercomputing Applications
##                		University of Illinois at Urbana-Champaign
##                		       http://www.ncsa.uiuc.edu
## 
## Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
## and associated documentation files (the "Software"), to deal in the Software without 
## restriction, including without limitation the rights to use, copy, modify, merge, publish, 
## distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the 
## Software is furnished to do so, subject to the following conditions: 
## 
##   - Redistributions of source code must retain the above copyright notice, this list of 
##     conditions and the following disclaimers. 
## 
##   - Redistributions in binary form must reproduce the above copyright notice, this list 
##     of conditions and the following disclaimers in the documentation and/or other 
##     materials provided with the distribution.
## 
##   - Neither the names of the University of Illinois, NCSA, nor the names of its 
##     contributors may be used to endorse or promote products derived from this 
##     Software without specific prior written permission. 
## 
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
## EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
## OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
## NONINFRINGEMENT. IN NO EVENT SHALL THE CONTIBUTORS OR 
## COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
## ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE 
## USE OR OTHER DEALINGS IN THE SOFTWARE.
## 
###########################################################################
##
## What:
##	NLANR Multicast BeaconServer, HTML Generator code
##
##	Version 0.8.09 - Last Edited: February 7, 2002, Mitch
##
##	January, 2002  - Mitch Kutzko
##	mitch@dast.nlanr.net | mitch@ncsa.uiuc.edu
##	Feedback to: beacon@dast.nlanr.net
##
## Filename and Version:
##	beaconserver_v0.8.09_htmlgen.pl
##
## Description:
##
##	Reads the intermediate format file "table2.out" from the 
##	current directory, and uses that data to generate the output 
##	HTML files every $WEBUPDATE seconds.  Those files must then 
##	be linked into a directory being served by a web server.
##
##	The tuples written to "table2.out" by the 
##	beaconserver_v0.8.09_server.pl script are written
##	IN THE ORDER THEY OCCUR IN IN THE PACKET -- Ie, delay, 
##	jitter, loss, order, duplicate!!
##
## Background:
##
##	This script is a Perl version of the original v0.63 Java 
##	NLANR Multicast Beacon, originally written by Kai Chen.  
##	See http://dast.nlanr.net/Projects/Beacon/.
##
## Required Modules:
## 	The perl modules listed in the 'use' statements at the beginning:
##	"IO::Socket", "IO::Socket::Multicast", and "Socket"
##
##	"TimeDate" will also need to be installed for this script to 
## 	function properly.
##
##
## Notes:
##
## The beaconserver_v0.8.09_htmlgen.pl script creates HTML output files from the 
## table2.out file created by the beaconserver_v0.8.09_server.pl script.
##
## In the interest of simplicity and scalability, the 
## BeaconServer really has no business pretending to be a web 
## server, and so it doesn't.
##
## Serving those files to the world is up to you.  We suggest running your favorite web
## server on the same machine as the BeaconServer, and linking the HTML files that the 
## BeaconServer generates into a directory underneath your web server's /docroot.
## Note that no "index.html" file is generated -- Typically, you just create an
## additional link to whichever set of data you want to display first as "index.html".
## For most folks, this means linking "loss.html" to "index.html" in the web directory.
##
## All files generated by these scripts are generated in the local directory where 
## the scripts are run.
##
## "table2.out" is the only output file actually used by this version of the
## BeaconServer -- "table.out" is the original format of the file, and is provided 
## here purely as a convenience for any other applications which may already use 
## that format.
##
## The user the web server runs as needs to have permissions to read the various files
## generated by these scripts.
##
## IMPORTANT NOTE:
## In this version, it is necessary to let the beaconserver.pl script run for 
## one $UPDATEINTERVAL prior to starting the htmlgenerator.pl script, else the 
## beaconserver_v0.8.09_htmlgen.pl script will fall over due to not finding the 
## table2.out file initially.  (This will be made more robust in a future version.)
##
##
## Acknowledgements:
## NLANR would like to specifically acknowledge the much-appreciated assistance of
## ANL's Bob Olson, NDSU's Marty Hoag, and the ACCESS Grid Community in the testing 
## and debugging of this script.
##
##
## This version of the Perl BeaconServer script was written by Mitch 
## Kutzko, NLANR/DAST, based on an earlier version of the script written 
## by Tony Rimovsky and Jon Dugan of NCSA Netdev (netdev@ncsa.uiuc.edu).      
##
## Mitch Kutzko, mitch@ncsa.uiuc.edu, November 2001
##
#########################################################################################
##

$VER 		= "0.8.09";	# Version number

$METADATALIMIT	= 6;	# Number of pieces of metadata per beacon
$CRBLIMIT       = 6;    # Number of things (-1 for the 0-slot) in Current Reporting Beacon info.

$WEBREFRESH	= 120;	# Update the pages every N seconds.
$COLHEADERS	= 10;	# Insert a line of column headers every N rows.
$ROWHEADERS	= 10;	# Insert a line of row headers every N columns.

$CELLWIDTH      = 40;   # How wide are the table cells?
$CELLHEIGHT     = 16;   # And how tall are they?


# Web page output color values
$lossgreen      = 10;   # %
$lossyellow     = 30;
$lossbogus      = 100;

$delaygreen     = 200;  # milliseconds
$delayyellow    = 1000;
$delaybogus     = 10000;

$jittergreen    = 50;   # milliseconds;
$jitteryellow   = 100;
$jitterbogus    = 10000;

$ordergreen     = 1;    # %
$orderyellow    = 5;
$orderbogus     = 100;

$duplicategreen = 1;    # %
$duplicateyellow = 5;
$duplicatebogus = 100;

use IO::Socket;

require "ctime.pl";
## Also requires that the 'TimeDate' module be installed, too...



#--------------------------------------------------------------
#--------------------------------------------------------------
# Web Output -- Generate the five data output HTML 
#   files: loss, delay, jitter, order, and duplicate
#--------------------------------------------------------------
#--------------------------------------------------------------

sub weboutput {
# Arg is name of file to open & output: "loss", "delay", "jitter", "duplicate", and "order".
$thispass = shift(@_);          # Gets the current argument and moves it into "$thispass"


# open the temporary outfile - Where we'll be writing HTML data off to prior to swapping it into the real file....
$outfile = ">$thispass\.TEMP";
open(OUTFILE, $outfile) || die ("OUTFILE open failed" );
                                   # leaving off the "\n" newline char
                                   # in the text of the -die- causes
                                   # it to print error info!

print OUTFILE "<HTML><HEAD>\n";
print OUTFILE "<TITLE>Beacon Webview for \"$thispass\" characteristics on $datetemp";

# Start the HTML file
print OUTFILE "</TITLE>\n";
print OUTFILE "<META HTTP-EQUIV=\"REFRESH\" CONTENT=\"$WEBREFRESH\">\n";
print OUTFILE "</HEAD><BODY>\n";
print OUTFILE "<TABLE WIDTH=\"100%\"><TR><TD ALIGN=\"BOTTOM\">\n";
print OUTFILE "<FONT SIZE=\"5\"><B>\
        <a href=\"http://dast.nlanr.net/Projects/Beacon/\">Multicast Beacon</a></B>\n";
print OUTFILE "</FONT>&nbsp;<FONT SIZE=\"2\">v$VER</FONT><br>\n";
print OUTFILE "Feedback to <a href=\"mailto:beacon\@dast.nlanr.net\">beacon\@dast.nlanr.net</a>\n";
print OUTFILE "</TD><TD>";
print OUTFILE "<FONT SIZE=\"2\">Color key:</TD>";

# Write color legend at the top of the page for this particular pass of data
if ($thispass eq "delay") {
        print OUTFILE "<TD BGCOLOR=\"lightgreen\"><FONT SIZE=\"2\">Green &lt;= $delaygreen ms Delay</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"yellow\"><FONT SIZE=\"2\">Yellow &lt;= $delayyellow ms Delay</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"red\"><FONT SIZE=\"2\">Red &lt;= $delaybogus ms Delay<BR>(*** = \
                impossible data)</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"gray\"><FONT SIZE=\"2\">Gray = No Data\n";
} elsif ($thispass eq "jitter") {
        print OUTFILE "<TD BGCOLOR=\"lightgreen\"><FONT SIZE=\"2\">Green &lt;= $jittergreen ms Jitter";
        print OUTFILE "<TD BGCOLOR=\"yellow\"><FONT SIZE=\"2\">Yellow &lt;= $jitteryellow ms Jitter";
        print OUTFILE "<TD BGCOLOR=\"red\"><FONT SIZE=\"2\">Red &lt;= $jitterbogus ms Jitter<BR>(*** = \
                impossible data)</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"gray\"><FONT SIZE=\"2\">Gray = No Data</TD>\n";
} elsif ($thispass eq "loss") {
        print OUTFILE "<TD BGCOLOR=\"lightgreen\"><FONT SIZE=\"2\">Green &lt;= $lossgreen % Loss</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"yellow\"><FONT SIZE=\"2\">Yellow &lt;= $lossyellow% Loss</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"red\"><FONT SIZE=\"2\">Red &lt;= $lossbogus% Loss<BR>(*** = \
                impossible data)</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"gray\"><FONT SIZE=\"2\">Gray = No Data</TD>\n";
} elsif ($thispass eq "order") {
        print OUTFILE "<TD BGCOLOR=\"lightgreen\"><FONT SIZE=\"2\">Green &lt;= $ordergreen% OOP</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"yellow\"><FONT SIZE=\"2\">Yellow &lt;= $orderyellow% OOP</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"red\"><FONT SIZE=\"2\">Red &lt;= $orderbogus% OOP<BR>(*** = \
                impossible data)</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"gray\"><FONT SIZE=\"2\">Gray = No Data</TD>\n";
} elsif ($thispass eq "duplicate") {
        print OUTFILE "<TD BGCOLOR=\"lightgreen\"><FONT SIZE=\"2\">Green &lt;= $duplicategreen% \
                Duplicate</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"yellow\"><FONT SIZE=\"2\">Yellow &lt;= $duplicateyellow% Duplicate</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"red\"><FONT SIZE=\"2\">Red &lt;= $duplicatebogus% Duplicate<BR>(*** = \
                impossible data)</TD>\n";
        print OUTFILE "<TD BGCOLOR=\"gray\"><FONT SIZE=\"2\">Gray = No Data</TD>\n";
}

print OUTFILE "</TR></TABLE>\n";

# Write the links to the other files
print OUTFILE "<P>\n";
print OUTFILE "<A HREF=\"loss.html\">Loss</A>&nbsp&nbsp&nbsp\n";
print OUTFILE "<A HREF=\"delay.html\">Delay</A>&nbsp&nbsp&nbsp\n";
print OUTFILE "<A HREF=\"jitter.html\">Jitter</A>&nbsp&nbsp&nbsp\n";
print OUTFILE "<A HREF=\"order.html\">Out of Order Packets</A>&nbsp&nbsp&nbsp\n";
print OUTFILE "<A HREF=\"duplicate.html\">Duplicate Packets</A>&nbsp;&nbsp;&nbsp;";
print OUTFILE "<A HREF=\"host.html\">Beacon Info</A> &nbsp;&nbsp;&nbsp; \
        [ <A HREF=\"table.out\">Original Table File</A> | [ <A HREF=\"table2.out\">Modified Table File</A> ]\n";
print OUTFILE "<BR>";

# Mine
#print OUTFILE "<A HREF=\"php_beacon/beacon.php\">Select a Subset of these Beacons</A>&nbsp&nbsp&nbsp\n";

# Bob's
print OUTFILE "<A HREF=\"beacon-php/beacon.php\">Select a Subset of these Beacons</A>&nbsp&nbsp&nbsp\n";

print OUTFILE "<BR>";

# Write the general information about this particular BeaconServer session
print OUTFILE "Time: <B>$datetemp</B> | ";
print OUTFILE "Page Refresh: <B>$WEBREFRESH seconds</B> | \n";
print OUTFILE "Beacons: <B> $totalbeacons </B><BR>\n";
print OUTFILE "Target Multicast Group: <B>$GROUP</B> | ";
print OUTFILE "Server listening on port: <B>$PORT</B><BR>\n";
print OUTFILE "<P>";

print OUTFILE "<P>\n<TABLE BORDER=\"1\">\n";

# Now plot all the data we've sorted
$linecount = 0;
foreach $myref (@sorted) {	# reference into the sorted array of data
	# Just the headers 
        if ($linecount%$COLHEADERS == 0) {      # How often to put in headers = $linecount modulo COLHEADERS

                # HEADERS - Print the row of sender headers across top
                print OUTFILE "<TR bgcolor=\"lightblue\">\n";
                if ($thispass eq "delay") {
                        print OUTFILE "<TH colspan=\"3\">Delay (ms)</TH>\n";
                } elsif ($thispass eq "jitter") {
                        print OUTFILE "<TH colspan=\"3\">Jitter (ms)</TH>\n";
                } elsif ($thispass eq "loss") {
                        print OUTFILE "<TH colspan=\"3\">Packet Loss (%)</TH>\n";
                } elsif ($thispass eq "order") {
                        print OUTFILE "<TH colspan=\"3\">Out of Order Packets(%)</TH>\n";
                } elsif ($thispass eq "duplicate") {
                        print OUTFILE "<TH colspan=\"3\">Duplicate Packets (%)</TH>\n";
                }

                for ($dd=0; $dd<$totalbeacons; $dd++) {
                        print OUTFILE "<TH width=\"$CELLWIDTH\" height=\"$CELLHEIGHT\">S$dd</TH>\n";
			if (($dd+1)%$ROWHEADERS == 0) {      # How often to put in headers = $linecount modulo ROWHEADERS
				print OUTFILE "<TD>&nbsp;</TD>\n";	# Hold the space in the table for the row headers
			}
                }
                print OUTFILE "</TR>\n";
        }

	# Ok, the actual data goes here!
	# Grab the current values of everything as it's due to be output(ed?)
	my $lname   = ${$myref}{NAME};
	my $lindex  = ${$myref}{INDEX};
	my $lhost   = ${$myref}{HOST};
	my $lsorthost=${$myref}{SORTHOST};
	my $lip     = ${$myref}{IP};
	my $lresolved=${$myref}{RESOLVED};
	my $losName = ${$myref}{OSNAME};
	my $losVer  = ${$myref}{OSVER};
	my $losArch = ${$myref}{OSARCH};
	my $lvmVer  = ${$myref}{VMVER};
	my $lbdata  = ${$myref}{BDATA};

	@tdata = split /[\/ ]/, $lbdata;		# Split the line into all (non-tupled) data values

        print OUTFILE "<TR>\n";

        # Now print this row for Receivers
        print OUTFILE "<TD bgcolor=\"#CCCCFF\"><B>R$linecount</B>&nbsp;</TD>\n";

	# update for 0.8.07 - Two columns for hostname/IP, host rj, ip lj.
        print OUTFILE "<TD bgcolor=\"#CCCCFF\" align=\"right\">";
	# update for 0.8.06 - Remove "beacon@" from the front of each line, just show hostname

	#print OUTFILE "$lname\@$lhost&nbsp;($lip)";
	@bold = reverse split /\./, $lhost;
	#print "$lhost) $#bold | @bold\n";	# DEBUG
	# update for 0.8.06 - Bold the last two chunks of the hostname if more than one chunk long
	#print OUTFILE "$lhost&nbsp;($lip)";
	if ($#bold > 1) {	# Three or more pieces to name
		for ($dd=$#bold; $dd>=0; $dd--) {	# Get each Beacon's data
			if ($dd < 2) { # bottom or next-to-bottom pieces - Bold 'em
				print OUTFILE "<b>$bold[$dd]</b>";	
			} else {
				print OUTFILE "$bold[$dd]";	
			}
			if ($dd > 0 ) {
				print OUTFILE ".";	
			}
		}
	} else {
		print OUTFILE "[<a href=\"http://dast.nlanr.net/Projects/Beacon/hostname_faq.html\">Click&nbsp;for&nbsp;FAQ</a>]&nbsp;&nbsp;&nbsp;$lhost";
	}
        print OUTFILE "</TD>\n";
	# update for 0.8.07 - Two columns for hostname/IP, host rj, ip lj.
        print OUTFILE "<TD bgcolor=\"#CCCCFF\" align=\"left\">";
	#print OUTFILE "&nbsp;$lip";
	print OUTFILE "$lip";
        print OUTFILE "</TD>\n";

	for ($count=0; $count<$totalbeacons; $count++) {
		if ($thispass eq "delay") {
			$color = "red";         # default to red
			$offset = (($newindex[$count] * 5) + 0);
			$data = $tdata[$offset];
			if ($data eq "-") {
				$color = "gray";
				print OUTFILE "<TD bgcolor=\"$color\">NA";
			} else {
				if (abs $data < $delaygreen) {
					$color = "lightgreen";
				} elsif (abs $data < $delayyellow) {
					$color = "yellow";
				}
				print OUTFILE "<TD bgcolor=\"$color\">";
				if (abs $data < $delaybogus) {
					print OUTFILE "$data";
				} else {        # Bogus data (Ie, > 10 seconds delay....)
					print OUTFILE "***";
				}
			}
		} elsif ($thispass eq "jitter") {
			$color = "red";         # default to red
			$offset = (($newindex[$count] * 5) + 1);
			$data = $tdata[$offset];
			if ($data eq "-") {
				$color = "gray";
				print OUTFILE "<TD bgcolor=\"$color\">NA";
			} else {
				if ($data < $jittergreen) {
					$color = "lightgreen";
				} elsif ($data < $jitteryellow) {
					$color = "yellow";
				}
				print OUTFILE "<TD bgcolor=\"$color\">";
				if (abs $data < $jitterbogus) {
					#print OUTFILE "$data";
					printf OUTFILE "%3.1f", $data;
				} else {        # Bogus data (Ie, > 10 seconds jitter....)
					print OUTFILE "***";
				}
			}
		}elsif ($thispass eq "loss") {
			$color = "red";         # default to red
			$offset = (($newindex[$count] * 5) + 2);
			$data = $tdata[$offset];
			if ($data eq "-") {
				$color = "gray";
				print OUTFILE "<TD bgcolor=\"$color\">NA";
			} else {
				if ($data < $lossgreen) {
					$color = "lightgreen";
				} elsif ($data < $lossyellow) {
					$color = "yellow";
				}
				print OUTFILE "<TD bgcolor=\"$color\">";
				if ($data <= $lossbogus) {
					print OUTFILE "$data";
				} else {        # Bogus data (Ie, > 100% loss....)
					print OUTFILE "***";
				}
			}
		} elsif ($thispass eq "order") {
			$color = "red";         # default to red
			$offset = (($newindex[$count] * 5) + 3);
			$data = $tdata[$offset];
			if ($data eq "-") {
				$color = "gray";
				print OUTFILE "<TD bgcolor=\"$color\">NA";
			} else {
                                if ($data < $ordergreen) {
                                        $color = "lightgreen";
                                } elsif ($data < $orderyellow) {
                                        $color = "yellow";
                                }
                                print OUTFILE "<TD bgcolor=\"$color\">";
                                if ($data <= $orderbogus) {
                                        print OUTFILE "$data";
                                } else {        # Bogus data (Ie, > 100% out of order packets....)
                                        print OUTFILE "***";
                                }
			}
		} elsif ($thispass eq "duplicate") {
			$color = "red";         # default to red
			$offset = (($newindex[$count] * 5) + 4);
			$data = $tdata[$offset];
			if ($data eq "-") {
				$color = "gray";
				print OUTFILE "<TD bgcolor=\"$color\">NA";
			} else {
                                if ($data < $duplicategreen) {
                                        $color = "lightgreen";
                                } elsif ($data < $duplicateyellow) {
                                        $color = "yellow";
                                }
                                print OUTFILE "<TD bgcolor=\"$color\">";
                                if ($data <= $duplicatebogus) {
                                        print OUTFILE "$data";
                                } else {        # Bogus data (Ie, > 100% duplicate packets....)
                                        print OUTFILE "***";
                                }
                        }
		}
		print OUTFILE "</TD>\n";

		# Column headers go here
		if (($count+1)%$ROWHEADERS == 0) {      # How often to put in headers = $linecount modulo ROWHEADERS
			print OUTFILE "<TD bgcolor=\"#CCCCFF\"><B>R$linecount</B></TD>\n";
		}
	}
	$linecount++;

	print OUTFILE "</TR>\n";
}
print OUTFILE "</TABLE>\n";
print OUTFILE "</BODY></HTML>\n";

close(OUTFILE);

# Swap the temp file into the real file.  "*Whump*!"
rename "$thispass\.TEMP", "$thispass\.html";

return();

}	# End of weboutput





#--------------------------------------------------------------
#--------------------------------------------------------------
# CRB Output -- Generate the info about the Currently Reporting Beacons
#--------------------------------------------------------------
#--------------------------------------------------------------

sub crboutput {

        # Print out the Beacon-specific info here.  Goes to $CRBLIMIT+1 to
        # get $session, which is tacked onto the end of crbinfo just to have
        # it here even though it's not part of the packet.
        #
        # CRB info: 0 = CRB name, 1 = ip, 2 = OS, 3 = OS ver, 4 = Arch, 5 = JVM


# Open the temporary file
$outfile = ">host\.TEMP";
open(OUTFILE, $outfile) || die ("OUTFILE open failed" );

# Start the HTML
print OUTFILE "<HTML><HEAD>\n";
print OUTFILE "<TITLE>Beacon Webview for Beacon Details on ";
print OUTFILE "$datetemp";

print OUTFILE "</TITLE>\n";
print OUTFILE "<META HTTP-EQUIV=\"REFRESH\" CONTENT=\"$WEBREFRESH\">\n";
print OUTFILE "</HEAD><BODY>\n";
print OUTFILE "<FONT SIZE=\"5\"><B>\
        <a href=\"http://dast.nlanr.net/Projects/Beacon/\">Multicast Beacon</a></B>";
print OUTFILE "</FONT>&nbsp;<FONT SIZE=\"2\">v$VER</FONT><br>\n";
print OUTFILE "Feedback to <a href=\"mailto:beacon\@dast.nlanr.net\">beacon\@dast.nlanr.net</a>\n";

print OUTFILE "<P>\n";

# Links to the other files
print OUTFILE "<A HREF=\"loss.html\">Loss</A>&nbsp&nbsp&nbsp\n";
print OUTFILE "<A HREF=\"delay.html\">Delay</A>&nbsp&nbsp&nbsp\n";
print OUTFILE "<A HREF=\"jitter.html\">Jitter</A>&nbsp&nbsp&nbsp\n";
print OUTFILE "<A HREF=\"order.html\">Out of Order Packets</A>&nbsp&nbsp&nbsp\n";
print OUTFILE "<A HREF=\"duplicate.html\">Duplicate Packets</A>&nbsp;&nbsp;&nbsp;";
print OUTFILE "<A HREF=\"host.html\">Beacon Info</A> &nbsp;&nbsp;&nbsp; \
        [ <A HREF=\"table.out\">Original Table File</A> | [ <A HREF=\"table2.out\">Modified Table File</A> ]\n";
print OUTFILE "<BR>";
# Mine
#print OUTFILE "<A HREF=\"php_beacon/beacon.php\">Select a Subset of these Beacons</A>&nbsp&nbsp&nbsp\n";

# Bob's
print OUTFILE "<A HREF=\"beacon-php/beacon.php\">Select a Subset of these Beacons</A>&nbsp&nbsp&nbsp\n";
print OUTFILE "<BR>";

# General information about this particular BeaconServer session
print OUTFILE "Time: <B>$datetemp</B> | ";
print OUTFILE "Page Refresh: <B>$WEBREFRESH seconds</B> | \n";
print OUTFILE "Beacons: <B> $totalbeacons </B><BR>\n";
print OUTFILE "Target Multicast Group: <B>$GROUP</B> | ";
print OUTFILE "Server listening on port: <B>$PORT</B><BR>\n";
print OUTFILE "<P>";

print OUTFILE "<P><TABLE BORDER=\"1\">\n";

# HEADERS - Print the row of sender headers across top
print OUTFILE "<TR bgcolor=\"blue\">\n";
print OUTFILE "<TH colspan=\"2\"><FONT COLOR=\"white\">Beacon</FONT></TH>\n";
print OUTFILE "<TH><FONT color=\"white\">IP</FONT></TH>\n";
print OUTFILE "<TH><FONT color=\"white\">Resolved As</FONT></TH>\n";
print OUTFILE "<TH><FONT color=\"white\">OS</FONT></TH>\n";
print OUTFILE "<TH><FONT color=\"white\">OS Version</FONT></TH>\n";
print OUTFILE "<TH><FONT color=\"white\">Arch</FONT></TH>\n";
print OUTFILE "<TH><FONT color=\"white\">JVM Version</FONT></TH>\n";
print OUTFILE "</TR>\n";

print OUTFILE "<TR>\n";

# Data for each reporting Beacon
$linecount = 0;
foreach $myref (@sorted) {	# reference into the sorted array of data

        # Grab the current values of everything as it's due to be output(ed?)
        my $lname   = ${$myref}{NAME};
        my $lindex  = ${$myref}{INDEX};
        my $lhost   = ${$myref}{HOST};
        my $lsorthost=${$myref}{SORTHOST};
        my $lip     = ${$myref}{IP};
        my $lresolved=${$myref}{RESOLVED};
        my $losName = ${$myref}{OSNAME};
        my $losVer  = ${$myref}{OSVER};
        my $losArch = ${$myref}{OSARCH};
        my $lvmVer  = ${$myref}{VMVER};
        my $lbdata  = ${$myref}{BDATA};

        for ($cc=0; $cc<$CRBLIMIT+1; $cc++) {		# "CRBLIMIT+1" to include the IP resolution data
                $crboutcolor = "lightgreen";            # Assume lightgreen
                if ($linecount%2 == 0) {                # Switch cell background colors for alternate cells
                        $crboutcolor = "lightblue";
                }

                if ($cc == 0) {                         # Print Beacon number in first cell only
                        print OUTFILE "<TD bgcolor=\"$crboutcolor\"><B>R$linecount</B>&nbsp;</TD>";
                }
		# Now write the information about each reporting Beacon
                # CRB info: 0 = CRB name, 1 = ip, 2 = OS, 3 = OS ver, 4 = Arch, 5 = JVM

		# Update for v8.08 - Bold last two chunks of domain name, and right justify
                print OUTFILE "<TD bgcolor=\"$crboutcolor\"";
		if ($cc == 0) {
			print OUTFILE " align=\"right\">";
		} else {
			print OUTFILE ">";
		}

                #if ($cc == 0) {print OUTFILE "$lname\@$lhost";}
		# Update for v8.08 - Bold last two chunks of domain name, and right justify
                if ($cc == 0) {
			@bold = reverse split /\./, $lhost;
			# update for 0.8.06 - Bold the last two chunks of the hostname if more than one chunk long
			if ($#bold > 1) {	# Three or more pieces to name
				print OUTFILE "$lname\@";
				for ($dd=$#bold; $dd>=0; $dd--) {	# Get each chunk of the beacon's Domain
					if ($dd < 2) { # bottom or next-to-bottom pieces - Bold 'em
						print OUTFILE "<b>$bold[$dd]</b>";	
					} else {
						print OUTFILE "$bold[$dd]";	
					}
					if ($dd > 0 ) {
						print OUTFILE ".";	
					}
				}
			} else {
				#print OUTFILE "$lhost";
				print OUTFILE "[<a href=\"http://dast.nlanr.net/Projects/Beacon/hostname_faq.html\">Click&nbsp;for&nbsp;FAQ</a>]&nbsp;&nbsp;&nbsp;$lname\@$lhost";
			}
		}
                if ($cc == 1) {print OUTFILE "$lip";}
                if ($cc == 2) {print OUTFILE "$lresolved";}
                if ($cc == 3) {print OUTFILE "$losName";}
                if ($cc == 4) {print OUTFILE "$losVer";}
                if ($cc == 5) {print OUTFILE "$losArch";}
                if ($cc == 6) {print OUTFILE "$lvmVer";}
                print OUTFILE "</TD>\n";
        }
        print OUTFILE "</TR>\n";

	$linecount++;
}

print OUTFILE "</TABLE>\n";
print OUTFILE "</BODY></HTML>\n";

close(OUTFILE);

# Swap the temp file to the real file.
rename "host\.TEMP", "host\.html";

return();

}	# End of crboutput









#--------------------------------------------------------------
#--------------------------------------------------------------
# Main routine begins here.....
#--------------------------------------------------------------
#--------------------------------------------------------------


my @names;				# Localize the array for the hash to do the sorting

$dateval = &ctime(time);       		# Get the current time
print "NLANR BeaconServer v$VER (Perl) - Generating HTML files from \"table2.out\" on $dateval";


while (1) {				# Loop forever, getting new data every $WEBREFRESH seconds

$tablefile = "< table2\.out";		# Open the table.out file for READING (the "< " in the open)
$tablefilename = "table2.out";

if (! -e $tablefilename ) {		# If the table.2out file doesn't exist yet, wait for it to show up
	print "No $tablefilename found yet -- Waiting for the server to create $tablefilename.";
	print " (This could take up to $WEBREFRESH seconds.)\n";
	while (! -e $tablefilename) {			# Waiting
		print "Waiting for $tablefilename.\n";
		sleep 10;
	}
	print "$tablefilename found!  HTML generator running now.\n";
}
open(TABLEFILE, $tablefile) || die ("TABLEFILE open failed" );


$line = <TABLEFILE>;			# read in first line, for the date
@temp = split / /, $line, 3;		# three chunks, from front, last chunk is actual date
chomp @temp;				# Whack the newlines from all entries in the array
$datetemp = $temp[2];

$line = <TABLEFILE>;			# read in second line, for the number of beacons
@temp = split / /, $line;		# "nbeacons <#>", get #
chomp @temp;				# Whack the newlines from all entries in the array
$totalbeacons = $temp[1];			# Get the current number of Beacons
chomp @temp;				# Whack the newlines from all entries in the array


$line = <TABLEFILE>;			# read in third line, for multicast group and port
@temp = split / /, $line;		# "group" <group#> "port" <port#>
$GROUP = $temp[1];
$PORT = $temp[3];

for ($ii=0; $ii<$totalbeacons; $ii++) {	# Get Beacon Names and IP addresses
	$line = <TABLEFILE>;		# get the next line
	@lineparts = split / /, $line;	# four parts -- <"beacon">, <#>, <beacon name>, <IP>
	chomp $lineparts[3];		# Whack the newline at the end of the IP address (it's last)
	@nameparts = split /@/, $lineparts[2];	# split name into BeaconName and HostName

	@tld = split /\./, $nameparts[1];# Split current name into component pieces
	foreach $kk (reverse @tld) {	# reverse the pieces for sorting
		$sorttld .= $kk . ".";	# assign $sorttld will the next chunk and a "."
	}
	if ( $#tld == 0 ) {		# Only one chunk to hostname, so not a real name
		$sorttld = "aaaaaaaa" . $sorttld;	# Force single chunk hostnames to one end of the list
	}

	chop $sorttld;			# Whack trailing "."

	push @name, $nameparts[0];	# Beacon Name (Typically "beacon")
	push @host, $nameparts[1];	# Hostname (lib-video.mcs.anl.gov)
	push @sorthost, $sorttld;	# Reversed hostname for sort order
	undef $sorttld;			# clear it for next pass through loop
	push @index, $lineparts[1];	# Where was this data originally?  ($lineparts[1] is 
						# original line # from "beacon # beaconname@www.xxx.yyy.edu"
	push @ip, $lineparts[3];	# save Beacon IP to Array

	# Get the "real" host name based on the supplied IP address and push to save array
	$targethost = gethostbyaddr(inet_aton($lineparts[3]), AF_INET);

	if (not defined $targethost) {	# Catch bogus or unresolvable values
		$tempresolve = "UNRESOLVABLE IP";
	} else {
		$tempresolve = lc($targethost);	# lowercase it for sorting
	}
	if ($lineparts[3] =~ m/^192.168/) {	# Catch NATed addresses (192.168.x.x)
			$tempresolve = "NATed Address";
	} elsif ($lineparts[3] =~m/^127.0.0/) {
			$tempresolve = "Localhost";
	}

	push @resolved, $tempresolve;	# Save resolved host to array
}

for ($ii=0; $ii<$totalbeacons; $ii++) {	# Get Report Data (stats) 
	$line = <TABLEFILE>;		# Get next line of actual Beacon report data
	@temp = split / /, $line, 2;	# ONLY TWO CHUNKS -- Used to throw away leading line number value
	chomp @temp;

	push @dataline, $temp[1];	# Save the line of data for later
}

for ($ii=0; $ii<$totalbeacons; $ii++) {	# Get Beacon Metadata (OS, OSVer, etc.) for all Beacons
	for ($jj=0; $jj<=$METADATALIMIT; $jj++) {	# Get each Beacon's data
		$line = <TABLEFILE>;		# Get the next line
		@temp =split / /, $line;	# Split the line into title and value
		chomp @temp;			# Gotta whack the newline at the end *HERE*

		if ( $jj < 3 ) {		# Skip beacon name, hostname, and IP, since we already have those.
		} elsif ( $jj == 3 ) {		# osName
			push @osName, $temp[1];
		} elsif ( $jj == 4 ) {		# osVer
			push @osVer, $temp[1];
		} elsif ( $jj == 5 ) {		# osArch
			push @osArch, $temp[1];
		} elsif ( $jj == 6 ) {		# vmVer
			push @vmVer, $temp[1];
		}
	}
}


close(TABLEFILE);	# Done with it here.....

for ($ii=0; $ii<$totalbeacons; $ii++) {		# Print everything

	# Build the hash for sorting all the data
	#my %hash = ( NAME => $name[$ii], INDEX => $index[$ii], HOST => $host[$ii], HOST1 => $tld[0], HOST2 => $tld[1], HOST3 => $tld[2], HOST4 => $tld[3], IP => $ip[$ii], RESOLVED => $resolved[$ii], OSNAME => $osName[$ii], OSVER => $osVer[$ii], OSARCH => $osArch[$ii], VMVER => $vmVer[$ii], BDATA => $dataline[$ii] );
	my %hash = ( NAME => $name[$ii], INDEX => $index[$ii], HOST => $host[$ii], SORTHOST => $sorthost[$ii], IP => $ip[$ii], RESOLVED => $resolved[$ii], OSNAME => $osName[$ii], OSVER => $osVer[$ii], OSARCH => $osArch[$ii], VMVER => $vmVer[$ii], BDATA => $dataline[$ii] );
	my $hashref =  \%hash;
	push @names, $hashref;
}




#----------------------------------------------------------------------------

# Sort it 
  sub sortByTLD {
    ${$a}{SORTHOST} cmp ${$b}{SORTHOST}
}
#    ${$a}{HOST4} cmp ${$b}{HOST4}
#    or ${$a}{HOST3} cmp ${$b}{HOST3}
#    or ${$a}{HOST2} cmp ${$b}{HOST2}
#    or ${$a}{HOST1} cmp ${$b}{HOST1}
#  }
  @sorted = sort sortByTLD @names;
  
  foreach $myref (@sorted) {
    my $lname	= ${$myref}{NAME};
    my $lindex	= ${$myref}{INDEX};
    my $lhost	= ${$myref}{HOST};
    my $lsorthost=${$myref}{SORTHOST};
    my $lip	= ${$myref}{IP};
    my $lresolved=${$myref}{RESOLVED};
    my $losName	= ${$myref}{OSNAME};
    my $losVer	= ${$myref}{OSVER};
    my $losArch	= ${$myref}{OSARCH};
    my $lvmVer	= ${$myref}{VMVER};
    my $lbdata	= ${$myref}{BDATA};

    push @newindex, $lindex;	# Save the original locations for the tuple data for the output sort

  }

#----------------------------------------------------------------------------------------------

# Actually create the HTML page(s) here.

weboutput("delay");
weboutput("jitter");
weboutput("loss");
weboutput("order");
weboutput("duplicate");

crboutput();		# Generate the HTML for the metadata about the beacons

# All output finished -- Clear arrays!
undef @sorted;
undef @name;
undef @host;
undef @sorthost;
undef @index;
undef @ip;
undef @resolved;
undef @dataline;
undef @names;
undef @osName;
undef @osVer;
undef @osArch;
undef @vmVer;
undef @newindex;


# Wait for $WEBREFRESH seconds before doing it all over again.
sleep $WEBREFRESH;

}			# Matches the very top "while (1)" statement for main loop

exit(0);
